import { Inject, Injectable, UnauthorizedException } from '@nestjs/common'
import * as bcrypt from 'bcrypt'
import { PrismaService } from 'nestjs-prisma'
import { Token, TokenPayload, TokenContnet } from './dto/token.dto'
import { JwtService } from '@nestjs/jwt'
import { securityConfig, SecurityConfig } from 'src/common/configs'

@Injectable()
export class TokenService {
  constructor(
    private jwtService: JwtService,
    private prismaService: PrismaService,
    @Inject(securityConfig.KEY)
    private secureConfig: SecurityConfig,
  ) {}

  async login(email: string, password: string) {
    const user = await this.prismaService.user.findUniqueOrThrow({
      where: { email },
    })

    const passwordValid = await bcrypt.compare(password, user.password)

    if (!passwordValid) {
      throw new UnauthorizedException('Invalid password')
    }

    return this.generateTokens({ userId: user.id })
  }

  async refreshToken(token: string) {
    const { userId, iat } = this.jwtService.verify<TokenContnet>(token, {
      secret: this.secureConfig.jwt_refresh_secret,
    })
    const user = await this.prismaService.user.findUniqueOrThrow({
      where: { id: userId },
    })
    if (iat * 1000 < user.updatedAt.getTime()) {
      throw new UnauthorizedException('token失效，请重新登录')
    }
    return {
      refreshToken: token,
      accessToken: this.jwtService.sign(
        { userId },
        {
          secret: this.secureConfig.jwt_refresh_secret,
          expiresIn: this.secureConfig.refreshIn,
        },
      ),
    }
  }

  generateTokens(payload: TokenPayload): Token {
    const accessToken = this.jwtService.sign(payload, {
      secret: this.secureConfig.jwt_access_secret,
      expiresIn: this.secureConfig.expiresIn,
    })
    const refreshToken = this.jwtService.sign(payload, {
      secret: this.secureConfig.jwt_refresh_secret,
      expiresIn: this.secureConfig.refreshIn,
    })

    return { accessToken, refreshToken }
  }
}
